﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using X3g.Plugin;

namespace SamplePlugin.ArticleHelper
{
    class CreateUserArticle
    {
        /// For convenience use the same names as the Plugin class does
        private DocumentManager Document;
        private IArticleManager X3gArticles;
        private IAppManager     X3gApp;
        
        /// Example Layer names
        ///
        /// The "_D3_" implies 3d geometry and the "_D2_" implies 2d geometry.
        /// The leading "72_" will enable them in the Layer Filter for the options:
        /// "Show 3D Layers", "Hide 3D Layers", "Show 2D Layers" and "Hide 2D Layers"
        ///
        private const string CustomLayerName_3_dimensional = "72_SampleVendor_SampleSeries_D3_";
        private const string CustomLayerName_2_dimensional = "72_SampleVendor_SampleSeries_D2_";


        /// <summary>
        /// Initialize relevant variables in the constructor.
        /// </summary>
        public CreateUserArticle(DocumentManager pDocumentManager, IArticleManager pArticleManager, IAppManager pAppManager)
        {
            Document = pDocumentManager;
            X3gArticles = pArticleManager;
            X3gApp = pAppManager;
        }

        /// <summary>
        /// Example on how to create an Article out of the currently selected Elements.
        /// </summary>
        public void FromSelection()
        {
            IList<DbEntity> allEntities = Document.GetSelection();

            if (allEntities.Count > 0)
            {
                DbBlockReference blockRef = Document.Blocks.Group(
                    allEntities, ""); Document.GetSelection();

                BuildArticle(blockRef);
            }
        }

        /// <summary>
        /// Example on how to create an Article interactive.
        /// </summary>
        public void Interactive()
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Filter = "Supported files |*.dwg;*.dxf;*.dwt;*.sat;*.3ds;*.fbx;" +
                "*.gltf;*.glb;*.stl;*.dae;*.skp;*.ifc;*.pec;|" +
                "All files |*.*";

            List<DbBlockReference> allEntities = new List<DbBlockReference>();

            // The following will set a default unit for imports without length units
            // but supress a window prompting for it.
            // Setting this option is not required to create article.
            IAppDialogOverride dialogOverride = new CustomDialogOverride();
            X3gApp.DialogOverride = dialogOverride;

            fileDialog.Title = I18N._("Select 2D Geometrics");
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                if (!Document.Layers.GetAllLayerNames().Contains(CustomLayerName_2_dimensional))
                    Document.Layers.CreateLayer(CustomLayerName_2_dimensional);

                DbBlockReference e = LoadEntities(fileDialog.FileName, CustomLayerName_2_dimensional);
                if (e != null)
                    allEntities.Add(e);
            }

            fileDialog.Title = I18N._("Select 3D Geometrics");
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                if (!Document.Layers.GetAllLayerNames().Contains(CustomLayerName_3_dimensional))
                    Document.Layers.CreateLayer(CustomLayerName_3_dimensional);

                DbBlockReference e = LoadEntities(fileDialog.FileName, CustomLayerName_3_dimensional);
                if (e != null)
                    allEntities.Add(e);
            }

            if (allEntities.Count == 0)
                return;

            DbBlockReference blockRef;
            if (allEntities.Count == 1)
                blockRef = allEntities[0];
            else
                blockRef = Document.Blocks.Group(allEntities, "");

            BuildArticle(blockRef);
        }

        /// <summary>
        /// Example on how to crate a simple UserArticle
        /// </summary>
        /// <param name="pBlockRef">The DbBlockReference containing 3D, 2D and additional information</param>
        private void BuildArticle(DbBlockReference pBlockRef)
        {
            IArticleEntity articleEntity = X3gArticles.CreateUserArticle(pBlockRef);
            IUserArticleInstance article = (IUserArticleInstance)X3gArticles.InstantiateArticle(articleEntity, null);

            article.SeriesId = "GX";
            article.ArticleNumber = "OFFICE-0123456789";
            article.ManufacturerId = "EG";
            article.PurchasePricePD = new Currency("USD", 300m);
            article.SalesPricePD = new Currency("EUR", 300m);
            article.ShortText = I18N._("Sample Article");
            article.LongText = I18N._("This is the long text for the article.");
            article.VariantText = I18N._("Variant Text.");

            // Set a custom data field with the key "EG-GX-CustomData"
            DbEntity.FromId(article.EntityId).CustomData.Set(
                "EG-GX-CustomData", I18N._("Default content"));
            article.Close();
        }

        /// <summary>
        /// Helper function to cover loading a file and adding a Layer to all imported entities.
        /// </summary>
        /// <param name="pPath">Path to the file to load.</param>
        /// <param name="pLayerName">(optional) Layer name to use.</param>
        /// <returns></returns>
        private DbBlockReference LoadEntities(string pPath, string pLayerName = null)
        {
            IList<DbEntity> tEntities = Document.InsertPlanning(pPath, false);
            if (tEntities.Count == 0)
                return null;

            DbBlockReference tReturn = Document.Blocks.Group(tEntities, string.Empty);
            if (!string.IsNullOrWhiteSpace(pLayerName))
            {
                tReturn.Layer = pLayerName;
            }
            return tReturn;
        }



        /// <summary>
        /// To avoid implementing the IAppDialogOverride interface from scratch, you can  
        /// extend the DefaultAppDialogOverride and overload the functionality you need.
        /// </summary>
        private class CustomDialogOverride : DefaultAppDialogOverride
        {
            public override bool GetInsertScale(GeBoundingBox3d pBounds, ref LengthUnit pUnit, ref double pCustomScale)
            {
                pUnit = LengthUnit.Meter;
                pCustomScale = 1.0;
                return (true);
            }
        }
    }
}
